import logging.config
import os
from dotenv import load_dotenv


class LogFactory:
    load_dotenv()
    log_dir = os.getenv('PYHURL_LOG_DIR', 'logs/')
    os.makedirs(log_dir, exist_ok=True)

    logging_configs = {
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'norm': {
                'format': '[%(asctime)s] [%(thread)d] [%(levelname)s] %(filename)s:%(lineno)d - %(message)s',
                'datefmt': '%Y-%m-%d %H:%M:%S'
            }
        },
        'handlers': {
            'console': {
                'level': 'DEBUG',
                'class': 'logging.StreamHandler',
                'formatter': 'norm'
            }
        },
        'loggers': {}
    }

    __loggers = {
    }

    logging.config.dictConfig(logging_configs)

    @classmethod
    def get_logger(cls, name='app', filename=None):
        if name in cls.logging_configs['loggers']:
            return logging.getLogger(name)

        cls.config_logger(name, filename)

        return logging.getLogger(name)

    @classmethod
    def config_logger(cls, name, filename):
        if filename is None:
            filename = os.path.join(cls.log_dir, name + '.log')
        cls.logging_configs['handlers'][name] = {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': filename,
            'maxBytes': 1024 * 1024 * 10,
            'backupCount': 3,
            'formatter': 'norm',
            'encoding': 'utf8'
        }
        cls.logging_configs['loggers'][name] = {
            'handlers': ['console', name],
            'level': 'INFO',
            'propagate': False
        }
        logging.config.dictConfig(cls.logging_configs)



from openai import OpenAI
import os
from dotenv import load_dotenv
import re
import json
from ollama import Client


class LLMClient:
    initiated = False

    def __init__(self, openai_api_key=None, openai_model=None, openai_api_base=None, ollama_host=None, ollama_model=None):
        load_dotenv()
        openai_key = os.getenv('PYHURL_OPENAI_API') if openai_api_key is None else openai_api_key
        self.openai_model = os.getenv('PYHURL_OPENAI_MODEL') if openai_model is None else openai_model
        openai_api_base = os.getenv('PYHURL_OPENAI_API_BASE') if openai_api_base is None else openai_api_base
        self.openai_client = OpenAI(api_key=openai_key, base_url=openai_api_base)

        ollama_host = os.getenv('PYHURL_OLLAMA_HOST', 'http://localhost:11434') if ollama_host is None else ollama_host
        self.ollama_model = os.getenv('PYHURL_OLLAMA_MODEL', 'llama2') if ollama_model is None else ollama_model
        self.ollama_client = Client(host=ollama_host)

    def call_openai(self, messages: list[dict], model=None, temperature=0.1, max_tokens=None, timeout=None):
        response = self.openai_client.chat.completions.create(
            model=self.openai_model if model is None else model,
            messages=messages,
            stream=False,
            temperature=temperature,
            max_tokens=max_tokens,
            timeout=timeout,
            tool_choice='auto',
            top_p=1)
        return response.choices[0].message.content

    def call_ollama(self, messages: list[dict], model=None, options=None):
        return self.ollama_client.chat(model=model if model else self.ollama_model, messages=messages, options=options)['message']['content']

    @classmethod
    def find_all_json_datas(cls, text):
        if not text:
            return []
        try:
            return [json.loads(text)]
        except:
            matches = re.findall(r'```(?:.*?)\s*([\[\{])(.*?)```', text, re.DOTALL)
            if matches:
                try:
                    return [json.loads(''.join(match)) for match in matches]
                except:
                    return []
            else:
                return []

    @classmethod
    def find_first_json_data(cls, text):
        datas = cls.find_all_json_datas(text)
        return datas[0] if len(datas) > 0 else None


import oss2
from dotenv import load_dotenv
import os
from pathlib import Path
import uuid

load_dotenv()


class OSSClient:
    access_key_id = os.getenv('PYHURL_OSS_ACCESS_KEY_ID', '')
    access_key_secret = os.getenv('PYHURL_OSS_ACCESS_KEY_SECRET', '')
    bucket_name = os.getenv('PYHURL_OSS_BUCKET_NAME', '')
    endpoint = os.getenv('PYHURL_OSS_ENDPOINT', '')

    auth = oss2.Auth(access_key_id, access_key_secret)
    bucket = oss2.Bucket(auth, endpoint, bucket_name)

    @classmethod
    def upload(cls, oss_filename: str, data) -> str:
        headers = dict()
        headers["x-oss-object-acl"] = oss2.OBJECT_ACL_PUBLIC_READ
        cls.bucket.put_object(oss_filename, data, headers=headers)
        return f"https://{cls.bucket_name}.{cls.endpoint.replace('https://', '')}/{oss_filename}"



import pymysql
import pymysql.cursors
from dotenv import load_dotenv
import os


class MySQLClient:
    load_dotenv()
    __conn_properties = {
        'host': os.getenv('PYHURL_MYSQL_HOST', 'localhost'),
        'port': int(os.getenv('PYHURL_MYSQL_PORT', 3306)),
        'database': os.getenv('PYHURL_MYSQL_DB'),
        'user': os.getenv('PYHURL_MYSQL_USER', 'root'),
        'password': os.getenv('PYHURL_MYSQL_PASSWORD'),
        'read_timeout': int(os.getenv('PYHURL_MYSQL_READ_TIMEOUT', 60)),
        'write_timeout': int(os.getenv('PYHURL_MYSQL_WRITE_TIMEOUT', 60)),
        'autocommit': bool(os.getenv('PYHURL_MYSQL_AUTO_COMMIT', False)),
        'cursorclass': pymysql.cursors.DictCursor,
    }

    @classmethod
    def get_connection(cls):
        return pymysql.connect(**cls.__conn_properties)

    @classmethod
    def close_connection(cls, conn):
        conn.close()

    @classmethod
    def execute(cls, sql, params=None, return_lastrowid=False):
        conn = cls.get_connection()
        try:
            with conn.cursor() as cursor:
                ret = cursor.execute(sql, params)
                lastrowid = cursor.lastrowid
            conn.commit()
            return lastrowid if return_lastrowid else ret
        finally:
            cls.close_connection(conn)

    @classmethod
    def fetchone(cls, sql, params=None):
        conn = cls.get_connection()
        try:
            with conn.cursor() as cursor:
                cursor.execute(sql, params)
                return cursor.fetchone()
        finally:
            cls.close_connection(conn)

    @classmethod
    def fetchmany(cls, sql, params=None):
        conn = cls.get_connection()
        try:
            with conn.cursor() as cursor:
                cursor.execute(sql, params)
                return cursor.fetchall()
        finally:
            cls.close_connection(conn)

    @classmethod
    def iterate(cls, sql, params=None):
        conn = cls.get_connection()
        try:
            with conn.cursor() as cursor:
                cursor.execute(sql, params)
                for row in cursor:
                    yield row
        finally:
            cls.close_connection(conn)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='pyhurl',
    version='1.0.7',
    description='A set of useful functions that I use in projects.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Rongliang Hu',
    author_email='hurongliang@gmail.com',
    packages=find_packages(),
    install_requires=[
        "pymysql",
        "python-dotenv",
        "oss2",
        "openai",
        "ollama"
    ],
    license='MIT',
    url='https://github.com/hurongliang/pyhurl',
    python_requires='>=3.8',
)


import os
import sqlite3
from abc import ABC
from typing import Optional
from dotenv import load_dotenv


class SqliteClient(ABC):
    load_dotenv()
    db_file = os.getenv('PYHURL_SQLITE_DB', 'db.sqlite3')

    @classmethod
    def __get_connection(cls):
        try:
            return sqlite3.connect(cls.db_file)
        except sqlite3.OperationalError:
            parent_folder = os.path.dirname(cls.db_file)
            if not os.path.exists(parent_folder):
                os.makedirs(parent_folder)
            return sqlite3.connect(cls.db_file)

    @classmethod
    def execute_script(cls, script):
        connection = cls.__get_connection()
        connection.executescript(script)
        connection.commit()
        connection.close()

    @classmethod
    def fetch_many(cls, sql, sql_params=None):
        """
        query db and return all rows
        """
        connection = cls.__get_connection()
        connection.row_factory = sqlite3.Row
        cursor = connection.cursor()
        if sql_params:
            rows = cursor.execute(sql, sql_params).fetchall()
        else:
            rows = cursor.execute(sql).fetchall()
        connection.close()
        return rows

    @classmethod
    def fetch_one(cls, sql, sql_params=None):
        """
        query db and return first row
        """
        rows = cls.fetch_many(sql, sql_params)
        return rows[0] if rows else None

    @classmethod
    def fetch_value(cls, sql, sql_params=None):
        """
        query db and return first element of first row
        """
        row = cls.fetch_one(sql, sql_params)
        return row[0] if row else None


    @classmethod
    def insert(cls, sql, params=None) -> Optional[int]:
        """
        insert into db and return last insert id
        """
        connection = cls.__get_connection()
        cursor = connection.cursor()
        cursor.execute("BEGIN;")
        try:
            if params:
                cursor.execute(sql, params)
            else:
                cursor.execute(sql)
            last_id = cursor.lastrowid
            connection.commit()
        except sqlite3.OperationalError as e:
            connection.rollback()
            raise e
        finally:
            connection.close()
        return last_id

    @classmethod
    def update(cls, sql, params=None) -> int:
        """
        update db and return affected rows
        """
        connection = cls.__get_connection()
        cursor = connection.cursor()
        cursor.execute("BEGIN;")
        try:
            if params:
                cursor.execute(sql, params)
            else:
                cursor.execute(sql)
            affected_rows = cursor.rowcount
            connection.commit()
        except sqlite3.OperationalError as e:
            connection.rollback()
            raise e
        finally:
            connection.close()
        return affected_rows
